import logging.config
import subprocess
from pathlib import Path
from typing import Optional
from typing import Union

from fzf.constants import ENCODING
from fzf.constants import MULTI_FLAG
from fzf.logging_ import logging_config
from fzf.utilities import resolve_input


PathLike = Union[bytes, str, Path]


logging.config.dictConfig(logging_config)
logger = logging.getLogger(__name__)


class FZF:
    """Class abstracting FZF command."""

    def __init__(
        self,
        executable: str = None,
        input: Optional[str] = None,
        cwd: PathLike = None,
        multi: bool = False,
        fzf_extras: list[str] = None,
    ) -> None:

        if executable is None:
            executable = 'fzf'
        if fzf_extras is None:
            fzf_extras = []

        self.fzf = executable
        self.input = input
        self.cwd: Optional[PathLike] = cwd
        self.multi: bool = multi

        # Extra Arguments
        self.fzf_args: list = fzf_extras

    def run(self, *args, **kwargs) -> Union[str, list[str]]:
        """Given current configuration, run fzf and return selection."""

        # Buid Command
        command: list[str] = [self.fzf]
        if self.multi:
            command.append(MULTI_FLAG)

        command += self.fzf_args

        input_: bytes = resolve_input(self.input)
        if input_:
            kwargs['stdin'] = subprocess.PIPE

        # Run FZF as Process
        process: subprocess.Popen = subprocess.Popen(
            command,
            *args,
            stdout=subprocess.PIPE,
            encoding=ENCODING,
            cwd=self.cwd,
            **kwargs,
        )  # type: ignore

        # Process - Input
        if input_:
            stdout, _ = process.communicate(input_.decode(ENCODING))
        else:
            stdout, _ = process.communicate()

        # Process - Output
        if self.multi:
            return stdout.splitlines()
        else:
            return stdout.strip()



from typing import Iterable
from typing import Optional
from typing import Union

from fzf.constants import ENCODING
from fzf.constants import FZFInputValues


def resolve_input(input_values: Optional[FZFInputValues]) -> bytes:
    """Resolve input values, for FZF, to bytes."""

    if not input_values:
        return b''

    if isinstance(input_values, bytes):
        return input_values

    if isinstance(input_values, str):
        return input_values.encode(ENCODING)

    if isinstance(input_values, Iterable):

        first_value: Union[str, bytes] = input_values[0]  # type: ignore

        if isinstance(first_value, bytes):
            return b'\n'.join(input_values)  # type: ignore
        elif isinstance(first_value, str):
            return '\n'.join(input_values).encode(ENCODING)  # type: ignore

    raise TypeError(f'Unsupported input type: {type(input_values)}')



from typing import Final
from typing import Iterable
from typing import Union


ENCODING: Final[str] = 'utf-8'
FZFInputValues = Union[bytes, str, Iterable[str], Iterable[bytes]]
MULTI_FLAG: Final[str] = '--multi'



import setuptools


with open("README.md", "r") as fin:
    long_description = fin.read()
with open("LICENSE", "r") as fin:
    license = fin.read()

setuptools.setup(
    name='fzflib',
    version='0.1.1',
    author='Kyle L. Davis',
    author_email='AceofSpades5757.github@gmail.com',
    url='https://github.com/AceofSpades5757/fzflib',
    description='A Python library for interacting with FZF.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    python_requires='>=3.6',
    packages=setuptools.find_packages('src', exclude=['tests']),
    package_dir={'': 'src'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)
